@@ -2,7 +2,7 @@
 #
 # Encoding of Arabic: ArabTeX Notation by Klaus Lagally ############################ 2003/06/19
 
-# $Id: ArabTeX.pm,v 1.18 2003/09/08 14:43:14 smrz Exp $
+# $Id: ArabTeX.pm,v 1.21 2004/01/12 20:20:50 smrz Exp $
 
 package Encode::Arabic::ArabTeX;
 
@@ -14,7 +14,7 @@ use warnings;
 use Scalar::Util 'blessed';
 use Carp;
 
-our $VERSION = do { my @r = q$Revision: 1.18 $ =~ /\d+/g; sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = do { my @r = q$Revision: 1.21 $ =~ /\d+/g; sprintf "%d." . "%02d" x $#r, @r };
 
 
 use Encode::Encoding;
@@ -293,7 +293,8 @@ sub decoder ($;$$) {
     my @scope = (
                     "'", "b", "t", "_t", "^g", ".h", "_h", "d", "_d", "r", "z", "s", "^s",
                     ".s", ".d", ".t", ".z", "`", ".g", "f", "q", "k", "l", "m", "n", "h",
-                    "w", "T", "|", "B"  # "y" treated specifically in some cases
+                    "w", "p", "v", "g", "c", "^c", ",c", "^z", "^n", "^l", ".r", "T",
+                    "|", "B",   # "y" treated specifically in some cases
                 );
 
 
@@ -382,7 +383,7 @@ sub decoder ($;$$) {
                     "U'",           "U'|",
 
                     "a'",           "a'a",
-                    "a'i",          "a'i",
+                    "a'i",          "a'ii",
                     "i'",           "i'y",
                     "u'",           "u'w",
 
@@ -785,7 +786,7 @@ sub decoder ($;$$) {
                     'silent' => 0,
                 ],
 
-                # modern internal substitution with wa.sla
+                # modern internal substitution with wa.sla .. lam + vowel + 'alif + vowel below
                 (
                     map {
 
@@ -793,7 +794,7 @@ sub decoder ($;$$) {
 
                         map {
 
-                            $vowel . "\x{0627}" . $_, [ $vowel, "\x{0671}" ],
+                            $vowel . "\x{0627}" . $_, $vowel . "\x{0671}",
 
                         } "\x{064E}", "\x{064F}", "\x{0650}"
 
@@ -846,6 +847,28 @@ sub decoder ($;$$) {
                     } "\x{0622}", "\x{0623}", "\x{0625}", "\x{0627}", "\x{0671}"
                 ),
 
+                # laam + vowel + 'alif + vowel .. internal substitution with wa.sla
+                (
+                    map {
+
+                        my $double = $_;
+
+                        map {
+
+                            my $vowel = $_;
+
+                            map {
+
+                                "\x{0644}" . $double . $vowel . "\x{0627}" . $_,
+                                "\x{0644}" . "\x{0671}" . $double . $vowel,
+
+                            } "\x{064E}", "\x{064F}", "\x{0650}"
+
+                        } "\x{064E}", "\x{064F}", "\x{0650}"
+
+                    } "", "\x{0651}"
+                ),
+
                 # optional ligatures to enforce here
 
             );
@@ -979,7 +1002,7 @@ Encode::Arabic::ArabTeX - Perl extension for multi-purpose processing of the Ara
 
 =head1 REVISION
 
-    $Revision: 1.18 $             $Date: 2003/09/08 14:43:14 $
+    $Revision: 1.21 $             $Date: 2004/01/12 20:20:50 $
 
 
 =head1 SYNOPSIS
@@ -1137,7 +1160,7 @@ Perl is also designed to make the easy jobs not that easy ;)
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003 by Otakar Smrz
+Copyright 2003, 2004 by Otakar Smrz
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
